@@ -1,21 +1,9 @@
 /*
    This file is part of the kimap library.
-   Copyright (C) 2007 Tom Albers <tomalbers@kde.nl>
-   Copyright (c) 2007 Allen Winter <winter@kde.org>
+   SPDX-FileCopyrightText: 2007 Tom Albers <tomalbers@kde.nl>
+   SPDX-FileCopyrightText: 2007 Allen Winter <winter@kde.org>
 
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
-   License version 2 as published by the Free Software Foundation.
-
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-
-   You should have received a copy of the GNU Library General Public License
-   along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.
+   SPDX-License-Identifier: LGPL-2.0-only
 */
 #include <QTest>
 
