@@ -1,20 +1,7 @@
 /*
-    Copyright (c) 2009 Kevin Ottens <ervin@kde.org>
+    SPDX-FileCopyrightText: 2009 Kevin Ottens <ervin@kde.org>
 
-    This library is free software; you can redistribute it and/or modify it
-    under the terms of the GNU Library General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or (at your
-    option) any later version.
-
-    This library is distributed in the hope that it will be useful, but WITHOUT
-    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
-    License for more details.
-
-    You should have received a copy of the GNU Library General Public License
-    along with this library; see the file COPYING.LIB.  If not, write to the
-    Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-    02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #ifndef KIMAP_IDLEJOB_H
@@ -57,7 +44,7 @@ class IdleJobPrivate;
  *
  * This job requires that the server supports the IDLE
  * capability, defined in
- * <a href="http://www.apps.ietf.org/rfc/rfc2177.html">RFC 2177</a>.
+ * <a href="https://tools.ietf.org/html/rfc2177">RFC 2177</a>.
  */
 class KIMAP_EXPORT IdleJob : public Job
 {
