@@ -6,44 +6,68 @@
 
 #include "selectjob.h"
 
-#include <KLocalizedString>
 #include "kimap_debug.h"
 
 #include "job_p.h"
 #include "response_p.h"
 #include "session_p.h"
 #include "rfccodecs.h"
+#include "imapset.h"
+
+#include <KLocalizedString>
+
+#include <QMap>
+#include <QTimer>
 
 namespace KIMAP
 {
 class SelectJobPrivate : public JobPrivate
 {
 public:
-    SelectJobPrivate(Session *session, const QString &name)
-        : JobPrivate(session, name), readOnly(false), messageCount(-1), recentCount(-1),
-          firstUnseenIndex(-1), uidValidity(-1), nextUid(-1), highestmodseq(0),
-          condstoreEnabled(false) { }
-    ~SelectJobPrivate() { }
+    SelectJobPrivate(SelectJob *q, Session *session, const QString &name)
+        : JobPrivate(session, name), q(q)
+    {
+        QObject::connect(&emitPendingsTimer, &QTimer::timeout, [this]() { emitPendings(); });
+    }
+
+    void emitPendings()
+    {
+        if (pendingMessages.empty()) {
+            return;
+        }
+
+        Q_EMIT q->modified(pendingMessages);
+        pendingMessages.clear();
+    }
 
     QString mailBox;
-    bool readOnly;
+    bool readOnly = false;
+
+    QMap<qint64, Message> pendingMessages;
+    QTimer emitPendingsTimer;
 
     QList<QByteArray> flags;
     QList<QByteArray> permanentFlags;
-    int messageCount;
-    int recentCount;
-    int firstUnseenIndex;
-    qint64 uidValidity;
-    qint64 nextUid;
-    quint64 highestmodseq;
-    bool condstoreEnabled;
+    int messageCount = -1;
+    int recentCount = -1;
+    int firstUnseenIndex = -1;
+    qint64 uidValidity = -1;
+    qint64 nextUid = -1;
+    quint64 highestmodseq = 0;
+    qint64 lastUidvalidity = -1;
+    quint64 lastModseq = 0;
+    ImapSet knownUids;
+
+    bool condstoreEnabled = false;
+
+    SelectJob * const q;
 };
 }
 
 using namespace KIMAP;
 
 SelectJob::SelectJob(Session *session)
-    : Job(*new SelectJobPrivate(session, i18nc("name of the select job", "Select")))
+    : Job(*new SelectJobPrivate(this, session, i18nc("name of the select job", "Select")))
 {
 }
 
@@ -135,6 +159,15 @@ bool SelectJob::condstoreEnabled() const
     return d->condstoreEnabled;
 }
 
+void SelectJob::setQResync(qint64 lastUidvalidity, quint64 lastModseq, const ImapSet &knownUids)
+{
+    Q_D(SelectJob);
+    d->lastUidvalidity = lastUidvalidity;
+    d->lastModseq = lastModseq;
+    d->knownUids = knownUids;
+    setCondstoreEnabled(true);
+}
+
 void SelectJob::doStart()
 {
     Q_D(SelectJob);
@@ -147,9 +180,20 @@ void SelectJob::doStart()
     QByteArray params = '\"' + KIMAP::encodeImapFolderName(d->mailBox.toUtf8()) + '\"';
 
     if (d->condstoreEnabled) {
-        params += " (CONDSTORE)";
+        // Check whether we only do CONDSTORE, or QRESYNC
+        if (d->lastUidvalidity == -1 && d->lastModseq == 0) {
+            params += " (CONDSTORE)";
+        } else {
+            params += " (QRESYNC (" + QByteArray::number(d->lastUidvalidity) + " "
+                                    + QByteArray::number(d->lastModseq);
+            if (!d->knownUids.isEmpty()) {
+                params += " " + d->knownUids.toImapSequenceSet();
+            }
+            params += "))";
+        }
     }
 
+    d->emitPendingsTimer.start(100);
     d->tags << d->sessionInternal()->sendCommand(command, params);
 }
 
@@ -166,6 +210,15 @@ void SelectJob::handleResponse(const Response &response)
         }
     }
 
+    // We can predict it'll be handled by handleErrorReplies() so stop
+    // the timer now so that result() will really be the last emitted signal.
+    if (!response.content.isEmpty() &&
+            d->tags.size() == 1 &&
+            d->tags.contains(response.content.first().toString())) {
+        d->emitPendingsTimer.stop();
+        d->emitPendings();
+    }
+
     if (handleErrorReplies(response) == NotHandled) {
         if (response.content.size() >= 2) {
             QByteArray code = response.content[1].toString();
@@ -201,15 +254,54 @@ void SelectJob::handleResponse(const Response &response)
                 }
             } else if (code == "FLAGS") {
                 d->flags = response.content[2].toList();
+            } else if (code == "VANISHED" && response.content.size() == 4) { // VANISHED response in SELECT is RFC5162 (QRESYNC) extension
+                const auto vanishedSet = ImapSet::fromImapSequenceSet(response.content[3].toString());
+                Q_EMIT vanished(vanishedSet);
             } else {
-                bool isInt;
+                bool isInt = false;
                 int value = response.content[1].toString().toInt(&isInt);
                 if (!isInt || response.content.size() < 3) {
                     return;
                 }
 
                 code = response.content[2].toString();
-                if (code == "EXISTS") {
+                if (code == "FETCH") { // FETCH response in SELECT is RFC5162 (QRESYNC) extension
+                    Message msg{};
+                    const auto content = response.content[3].toList();
+                    for (auto it = content.cbegin(), end = content.cend(); it != end; ++it) {
+                        const auto name = *it;
+                        ++it;
+
+                        if (it == content.constEnd()) {   // Uh oh, message was truncated?
+                            qCWarning(KIMAP_LOG) << "SELECT reply got truncated, skipping.";
+                            break;
+                        }
+
+                        if (name == "UID") {
+                            msg.uid = it->toLongLong();
+                        } else if (name == "FLAGS") {
+                            if ((*it).startsWith('(') && (*it).endsWith(')')) {
+                                QByteArray str = *it;
+                                str.chop(1);
+                                str.remove(0, 1);
+                                const auto flags = str.split(' ');
+                                msg.flags = flags;
+                            } else {
+                                msg.flags << *it;
+                            }
+                        } else if (name == "MODSEQ") {
+                            QByteArray modseq = *it;
+                            if (modseq.startsWith('(') && modseq.endsWith(')')) {
+                                modseq.chop(1);
+                                modseq.remove(0, 1);
+                            }
+
+                            msg.attributes.insert(name, modseq.toULongLong());
+                        }
+                    }
+
+                    d->pendingMessages.insert(value, msg);
+                } else if (code == "EXISTS") {
                     d->messageCount = value;
                 } else if (code == "RECENT") {
                     d->recentCount = value;
