@@ -27,13 +27,11 @@ static const int _kimap_sslErrorUiData = qRegisterMetaType<KSslErrorUiData>();
 }
 
 SessionThread::SessionThread(const QString &hostName, quint16 port)
-    : QObject(), m_hostName(hostName), m_port(port),
-      m_encryptedMode(false),
-      m_useProxy(false)
+    : QObject(), m_hostName(hostName), m_port(port)
 {
     // Just like the Qt docs now recommend, for event-driven threads:
     // don't derive from QThread, create one directly and move the object to it.
-    QThread *thread = new QThread();
+    auto *thread = new QThread();
     moveToThread(thread);
     thread->start();
     QMetaObject::invokeMethod(this, &SessionThread::threadInit);
